%                         VESSEL SORT

% This script sorts the data into specific data sets for each shuttle tanker
%
% The script generates a table "res2" which includes the all the results




clear all
tic

%% Import data from text file.

% Auto-generated by MATLAB on 2017/04/25 19:30:41

% Copyright 2014 The MathWorks, Inc.
% Automating File Import
% Select folder containing data interactively

L = 0;

Location =  'C:\Users\langd\Documents\Skole\Master\Data - Kystverket\Thesis material';
% Identify where to search for files
% Store the name of all .xls files as a vector D
D = dir([Location, '\*.csv']);
% Extract the file names
filenames = {D(:).name}.';



for y = 1:length(filenames)
    

    %% Initialize variables.
    filename = filenames{y};
    delimiter = ';';
    startRow = 2;

    %% Format for each line of text:
    formatSpec = '%f%f%q%q%f%f%f%f%q%f%f%f%f%q%[^\n\r]';

    %% Open the text file.
    fileID = fopen([Location '\' filename],'r');

    %% Read columns of data according to the format.
    % This call is based on the structure of the file used to generate this
    % code. If an error occurs for a different file, try regenerating the code
    % from the Import Tool.
    dataArray = textscan(fileID, formatSpec, 'Delimiter', delimiter, 'EmptyValue',NaN,'HeaderLines' ,startRow-1, 'ReturnOnError', false, 'EndOfLine', '\r\n');

    %% Close the text file.
    fclose(fileID);

    %% Create output variable
    shuttletankers = table(dataArray{1:end-1}, 'VariableNames', {'mmsi','imonumber','vesselname','date_time_utc','fuelconsumption','co2emission','utseilt_dist_nm','sec_nextpoint','l5_lloydstypename','sizegroupgrosston','utredningsomraadeid','long','lat','geom4326'});

    %% Finds all unique vessels
    %% 'imonumber' can be changed to 'mmsi'

    A = unique(shuttletankers(:,2));
    vesselid = A{1:size(A,1),{'imonumber'}};
    imonumber = shuttletankers{1:size(shuttletankers,1),{'imonumber'}};

    for i = 1:size(vesselid,1)
        res{i + L,1} = vesselid(i);
        res{i + L,2}  = shuttletankers(shuttletankers.imonumber == vesselid(i),:);
    end
    L = L + size(vesselid,1);
    
    for k =1:size(res,1)
        T(k,1) = res{k,1};
    end
    
    vessels = unique(T);
    numberofvessels = size(vessels,1);
end

%%
%% Sort the data according to time
for ii = 1:numberofvessels
    
    waitbar(ii/numberofvessels)
    
    %% Finds all the data positions for a vessel
    pos = [find(T(:)== vessels(ii))]; 
    res2(ii).results=[];
    for j=1:size(pos,1)
        res2(ii).results = [res2(ii).results; res{pos(j),2}];
    end
    
    %% Sort table according to time
    res2(ii).results = sortrows(res2(ii).results,4);
    res2(ii).name = res2(ii).results{1,3};
    

    newtable = res2(ii).results;
    
    %% Calculate fuel consumption, CO2 emissions and Speed (per minute)
    
    for p = 0:2:4
       for jj = 1:size(newtable,1)
           Z(jj,1) = newtable{jj,5+p}/(newtable{jj,8+(p/2)}/60);
       end
       
       Z = array2table(Z);
       
       newtable = [newtable(:,1:5+p) Z newtable(:,(6+p):(14+(p/2)))];
       
       if p == 0
           newtable.Properties.VariableNames{6 + p} = 'fuelpermin';
       elseif p == 2
           newtable.Properties.VariableNames{6 + p} = 'CO2permin';
       elseif p == 4
           newtable.Properties.VariableNames{6 + p} = 'distpermin';
       end
       
       Z = [];
    end
    
    res2(ii).results = newtable;
end


%% Initial Operationa mode set to blank = ''
for d = 1:size(res2,2)
    for s = 1:size(res2(d).results,1)
        res2(d).PosInfo(s).Operation = '';
    end
end

%% Estimate parameters for missing AIS data

elip = referenceEllipsoid('earth');
totalNANtime = 0;
for i = 1:size(res2,2)
 
    
    for j = 1:size(res2(i).results,1)
        if isnan(table2array(res2(i).results(j,11))) %if the time until next message is missing
           
            
            %% calculate the time between NAN message and next message
            res2(i).results(j,11) = {seconds(datetime(table2array(res2(i).results((j+1),4)))...
                - datetime(table2array(res2(i).results((j),4))))};
            %% Assume same speed and emissions as last message
            res2(i).results(j,10) = res2(i).results(j-1,10);
            res2(i).results(j,9) = {table2array(res2(i).results(j,11))*table2array(res2(i).results(j,10))/60};
            %% CO2 emisssions
            res2(i).results(j,8) =  res2(i).results(j-1,8);
            res2(i).results(j,7) = {table2array(res2(i).results(j,11))*table2array(res2(i).results(j,8))/60};
            %% Fuel consumption
            res2(i).results(j,6) =  res2(i).results(j-1,6);
            res2(i).results(j,5) = {table2array(res2(i).results(j,11))*table2array(res2(i).results(j,6))/60};
            
            
          
        end
    end
end

toc
